/***********************************************************************

    This source code is provided as is and may be used to extend 
    the functionality if desired. Do not remove all refernces to the
    original author.

    Ken Schirmeister
    KenSch@MountainCable.net  

***********************************************************************/

// MacroData.cpp: implementation of the CMacroData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TrivialTools.h"
#include "DatePrinter.h"
#include "MacroData.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////

const int    CMacroData::sm_iCnt = 10;

Field_t CMacroData::sm_aMacros[sm_iCnt] = 
{
    { "Date modified", Date , Modified , "DATE_MODIFIED"} ,
    { "Date created",  Date , Created  , "DATE_CREATED"} ,
    { "Date accessed", Date , Accessed , "DATE_ACCESSED"} ,
    { "Current Date",  Date , Current  , "DATE_CURRENT"} ,

    { "Time modified",  Time , Modified , "TIME_MODIFIED"} ,
    { "Time created",   Time , Created  , "TIME_CREATED"} ,
    { "Time accessed",  Time , Accessed , "TIME_ACCESSED"} ,
    { "Current Time",   Time , Current  , "TIME_CURRENT"} ,

    { "Current File",   FileName, Name,  "FILENAME"},
    { "Current User",   UserName, Name,  "USER"},
};


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/*static*/ void __fastcall CMacroData::ReadTexts(
        FILE *fp)       // mu bereits auf die richtige Stelle in der Datei
                        // positioniert sein.
{
    for ( int i = 0 ; i < sm_iCnt ; i++ )
        ReadString(fp, sm_aMacros[i].szField, numof(sm_aMacros[0].szField));
}

//////////////////////////////////////////////////////////////////////

const TCHAR *CMacroData::FindMacroText(  // get "Date modified", return "DATE_MODIFIED"
        const TCHAR *pszField)
{
    for ( int i = 0 ; i < sm_iCnt ; i++ )
        if ( strncmp(pszField, sm_aMacros[i].szField, strlen(sm_aMacros[i].szField)) == 0 )
            return (sm_aMacros[i].pszMacroText);

    return NULL;
}

//////////////////////////////////////////////////////////////////////

const TCHAR *CMacroData::FindTextForDlg(  // get "DATE_MODIFIED", return "Date modified"
        const TCHAR *pszMacro)
{
    for ( int i = 0 ; i < sm_iCnt ; i++ )
        if ( strncmp(pszMacro, sm_aMacros[i].pszMacroText, strlen(sm_aMacros[i].pszMacroText)) == 0 )
            return (sm_aMacros[i].szField);

    return NULL;
}

//////////////////////////////////////////////////////////////////////

static void __fastcall CMacroData::ReadTexts(
        FILE *fp);      // mu bereits auf die richtige Stelle in der Datei
                        // positioniert sein.

//////////////////////////////////////////////////////////////////////

bool __stdcall CMacroData::InsertMacro(  // true --> okay
        const TCHAR *szInput,            // zeigt auf ein Macro wie $(*DATE-CREATED %#x*)
        CString *pstrOutput,             // "Wed, June 16, 2003" wird angehngt
        const TCHAR *pszFileName,        // Name der MegaCAD-Datei
        int     *piSrc)
{
    bool bFoundIt = false;
    bool bOK = true;
    int iIndexSrc = *piSrc;
    // die srsten drei Zeichen berspringen (sind sowieso "$(*" )
    szInput += 3;
    iIndexSrc += 3;
    for ( int i = 0 ; i < sm_iCnt && !bFoundIt && bOK ; i++ )
    {
        if ( strncmp(szInput, sm_aMacros[i].pszMacroText, strlen(sm_aMacros[i].pszMacroText)) == 0 )
        {
            // gefunden!
            TCHAR szFormat[128];    // sehr grozgig!
            int iFormat = 0;
            iIndexSrc += strlen(sm_aMacros[i].pszMacroText);
            szInput += strlen(sm_aMacros[i].pszMacroText);

            // FormatString extrahieren: (sowas wie "%#x")
            while ( *szInput && *szInput != '*' )
            {
                if (*szInput != ' ')
                    szFormat[iFormat++] = *szInput;
                iIndexSrc++;
                szInput++;
            }

            bOK = false;
            // evtl vorhandene Blanks und die *) am Ende berspringen:
            while ( *szInput && *szInput == ' ' )
            {
                iIndexSrc++;
                szInput++;
            }

            // dann MUESSEN "*)" folgen:
            if ( *szInput && *szInput == '*' )
            {
                iIndexSrc++;
                szInput++;
                if ( *szInput && *szInput == ')' )
                {
                    szInput++;
                    iIndexSrc++;
                    bOK = true;
                }
            }

            ASSERT (bOK);

            if ( bOK)
            {
                szFormat[iFormat] = '\0';

                TCHAR szBuffer[128];

                m_TimeGetter.GetTime(
                                sm_aMacros[i].iTimeType,
                                szFormat,
                                pszFileName,
                                szBuffer);

                *pstrOutput += szBuffer;
                bFoundIt = true;
            }
        }
    }
    if ( bFoundIt && bOK )
        *piSrc = iIndexSrc;

    return bFoundIt && bOK;
}


